package com.isu.gaswellwatch.exception;

import com.isu.gaswellwatch.entity.Response;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.multipart.MaxUploadSizeExceededException;

import java.net.ConnectException;
import java.util.stream.Collectors;

@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    /**
     * 拦截自定义业务异常
     **/
    @ExceptionHandler(value = BusinessException.class)
    public Response<?> baseErrorHandler(HttpServletRequest request, BusinessException e) {
        log.error("【请求接口地址:{}】 业务逻辑错误信息:{}", request.getRequestURI(), e.getMessage());
        return Response.failed(e.getCode(), e.getMessage());
    }

    /**
     * 拦截微服务调用失败异常
     **/
    @ExceptionHandler(value = OpenFeignCallException.class)
    public Response<?> openFeignCallErrorHandler(HttpServletRequest request, OpenFeignCallException e) {
        log.error("请求接口地址:{}", request.getRequestURI());
        log.error("异常类:{}", e.getClass().getName());
        log.error("微服务调用异常信息如下:\n {}", ExceptionUtils.getStackTrace(e));
        return Response.failed(e.getCode(), e.getMessage());
    }

    /**
     * 拦截未知异常
     **/
    @ExceptionHandler(value = Exception.class)
    public Response<?> exceptionHandler(HttpServletRequest request, Exception e) {
        log.error("请求接口地址:{}", request.getRequestURI());
        log.error("异常类:{}", e.getClass().getName());
        log.error("异常堆栈信息如下:\n {}", ExceptionUtils.getStackTrace(e));
        if (e instanceof MethodArgumentNotValidException) {
            BindingResult bindingResult = ((MethodArgumentNotValidException) e).getBindingResult();
            if (bindingResult.hasErrors()) {
                String msg = bindingResult.getFieldErrors().stream().map(FieldError::getDefaultMessage)
                        .collect(Collectors.joining("<br>"));
                return Response.failed(msg);
            }
            return Response.failed(e.getMessage());
        } else if (e instanceof ConnectException) {
            return Response.failed("服务请求错误！");
        } else if (e instanceof BusinessException) {
            return Response.failed(e.getMessage());
        }  else if (e instanceof MaxUploadSizeExceededException) {
            return Response.failed("上传文件大小超过最大限制");
        }  else {
            return Response.failed("系统异常！");
        }
    }

    /**
     * 拦截所有controller方法中所有参数加了
     * <code>@Valid</code>注解的方法
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public Response<String> handlerParamErrorException(HttpServletRequest request, MethodArgumentNotValidException e) {
        BindingResult result = e.getBindingResult();
        String message = result.getFieldErrors().get(0).getDefaultMessage();
        log.error("【请求接口地址:{}】触发异常类:{} 参数错误信息:{}", request.getRequestURL().toString(), e.getClass(), message);
        return Response.failed(message);
    }

    /**
     * 拦截前端用户乱输入，引起的参数类型不匹配
     */
    @ExceptionHandler(value = MethodArgumentTypeMismatchException.class)
    public Response<String> handlerMethodArgumentTypeMismatchException(MethodArgumentTypeMismatchException e) {
        log.error("name:{}, propertyName:{}, message:{}, parameter:{}, localizedMessage:{}, value:{}",
                e.getName(), e.getPropertyName(), e.getMessage(),
                e.getParameter(), e.getLocalizedMessage(),
                e.getValue()
        );
        return Response.failed("【参数名】:" + e.getName() + "参数类型不匹配");
    }

    /**
     * 拦截参数转换异常
     */
    @ExceptionHandler(value = HttpMessageNotReadableException.class)
    public Response<String> handlerHttpMessageNotReadableException(HttpServletRequest request, HttpMessageNotReadableException e) {
        log.error("【请求接口地址:{}】触发异常类:{} 参数转换:{}", request.getRequestURI(), e.getClass(), e.getMessage());
        return Response.failed("参数类型不匹配");
    }

    /**
     * 拦截请求方法错误异常
     */
    @ExceptionHandler(value = HttpRequestMethodNotSupportedException.class)
    public Response<String> handlerHttpRequestMethodNotSupportedException(HttpServletRequest request, HttpRequestMethodNotSupportedException e) {
        log.error("请求接口地址:{},触发异常类:{} 请求方法不支持", request.getRequestURI(), e.getClass());
        return Response.failed("不支持的请求方法");
    }

    /**
     * 拦截 Http Servlet请求缺少参数异常，进行友好返回给前端
     */
    @ExceptionHandler(value = MissingServletRequestParameterException.class)
    public Response<String> handlerMissingServletRequestParameterException(HttpServletRequest request, MissingServletRequestParameterException e) {
        String translationMsg = "【参数名】：".concat(e.getParameterName())
                .concat("，【参数类型】：").concat(e.getParameterType())
                .concat("，【错误消息】：").concat("请求缺少该参数");
        String rawMsg = e.getLocalizedMessage();
        log.error("请求接口地址:{},触发异常类:{}, 异常消息:{}", request.getRequestURI(), e.getClass(), rawMsg);
        return Response.failed(translationMsg);
    }
}
